#pragma once

#include "Exception.h"

#include <memory>

namespace digidoc
{

EXP_DIGIDOC std::string appInfo();
EXP_DIGIDOC void initialize(const std::string &appInfo = "");
EXP_DIGIDOC void terminate();
EXP_DIGIDOC std::string version();

class Document;
class Signature;
class Signer;

class EXP_DIGIDOC ADoc
{
public:
	enum DocumentType
	{
		UnknownType = 0,
		BDocType = 1,
		DDocType = 2,
	};

	virtual ~ADoc() {}
	virtual void save() throw(IOException, BDocException) = 0;
	virtual void saveTo(const std::string &path) throw(IOException, BDocException) = 0;
	virtual void addDocument(const Document& document) throw(BDocException) = 0;
	virtual Document getDocument(unsigned int id) const throw(BDocException) = 0;
	virtual void removeDocument(unsigned int id) throw(BDocException) = 0;
	virtual unsigned int documentCount() const = 0;
	virtual DocumentType documentType() const;
	virtual void addSignature(const std::vector<unsigned char> &signature) throw(BDocException) = 0;
	virtual const Signature* getSignature(unsigned int id) const throw(BDocException) = 0;
	virtual void removeSignature(unsigned int id) throw(BDocException) = 0;
	virtual unsigned int signatureCount() const = 0;
	virtual void sign(Signer* signer) throw(BDocException) = 0;
	virtual std::vector<unsigned char> getFileDigest( unsigned int id ) throw(BDocException) = 0;
};

}
