#pragma once

#include "ADoc.h"
#include "Signature.h"

namespace digidoc
{

class DDocPrivate;
class SignatureDDOCPrivate;

/**
 * DDoc Signature class
 */
class EXP_DIGIDOC SignatureDDOC: public Signature
{
public:
	virtual ~SignatureDDOC();

	virtual std::string getMediaType() const;
    std::vector<unsigned char> getNonce() const;
	X509Cert getOCSPCertificate() const;
	std::string getProducedAt() const;
	std::string getResponderID() const;
	void getRevocationOCSPRef(std::vector<unsigned char>& data, std::string& digestMethodUri) const throw(SignatureException);
	virtual void validateOffline() const throw(SignatureException);
	virtual CertStatus validateOnline() const throw(SignatureException);

protected:
	virtual void sign(Signer* signer) throw(SignatureException, SignException);

private:
	SignatureDDOC( int id, DDocPrivate *doc );

	SignatureDDOCPrivate *d;

	friend class DDocPrivate;
};

/**
 * Implements the DDOC specification of the signed digital document container.
 * Container can contain several files and all these files can be signed using
 * signing certificates. Container can only be signed if it contains documents.
 * Documents can be added and removed from container only if the container is
 * not signed. To add or remove documents from signed container remove all the
 * signatures before modifying documents list in container.
 */
class EXP_DIGIDOC DDoc: public ADoc
{
public:
	DDoc();
	DDoc(const std::string &path) throw(IOException, BDocException);
	virtual ~DDoc();

	void save() throw(IOException, BDocException);
	void saveTo(const std::string &path) throw(IOException, BDocException);
	void addDocument(const Document& document) throw(BDocException);
	Document getDocument(unsigned int id) const throw(BDocException);
	void removeDocument(unsigned int id) throw(BDocException);
	unsigned int documentCount() const;
	DocumentType documentType() const;
	void addSignature(const std::vector<unsigned char> &signature) throw(BDocException);
	const Signature* getSignature(unsigned int id) const throw(BDocException);
	void removeSignature(unsigned int id) throw(BDocException);
	unsigned int signatureCount() const;
	void sign(Signer* signer) throw(BDocException);
	std::vector<unsigned char> getFileDigest( unsigned int id ) throw(BDocException);

private:
	DDocPrivate *d;
	void loadFile();
};

}
