#if !defined(__DIGEST_H_INCLUDED__)
#define __DIGEST_H_INCLUDED__

#include "../Exception.h"

#include <memory>

#define URI_SHA1 "http://www.w3.org/2000/09/xmldsig#sha1"
#define URI_SHA224 "http://www.w3.org/2001/04/xmldsig-more#sha224"
#define URI_SHA256 "http://www.w3.org/2001/04/xmlenc#sha256"
#define URI_SHA384 "http://www.w3.org/2001/04/xmldsig-more#sha384"
#define URI_SHA512 "http://www.w3.org/2001/04/xmlenc#sha512"

#define URI_RSA_SHA1 "http://www.w3.org/2000/09/xmldsig#rsa-sha1"
#define URI_RSA_SHA224 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha224"
#define URI_RSA_SHA256 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256"
#define URI_RSA_SHA384 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha384"
#define URI_RSA_SHA512 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512"

namespace digidoc
{
    /**
     * Digest calculation interface.
     */
    class DigestPrivate;
    class EXP_DIGIDOC Digest
    {
      public:
          Digest() throw(IOException);
          Digest(int method) throw(IOException);
          Digest(const std::string &methodUri) throw(IOException);
          virtual ~Digest();
          void reset();
          void update(std::vector<unsigned char> data) throw(IOException);
          virtual void update(const unsigned char *data, unsigned long length) throw(IOException);
          virtual std::vector<unsigned char> getDigest() throw(IOException);
          unsigned int getSize() const;
          int getMethod() const;
          std::string getName() const;
          std::string getUri() const;

          static int toMethod(const std::string &methodUri) throw(IOException);
          static bool isSupported(const std::string &methodUri);

      private:
          DigestPrivate *d;
    };

}

#endif // !defined(__DIGEST_H_INCLUDED__)
