#if !defined(__X509CERT_H_INCLUDED__)
#define __X509CERT_H_INCLUDED__

#include "../../Exception.h"

#ifdef WIN32 //hack for win32 build
#undef X509_NAME
#endif
#include <openssl/x509.h>

namespace digidoc
{
    /**
     * Wrapper for OpenSSL X509 certificate structure.
     */
    class EXP_DIGIDOC X509Cert
    {

      public:
          enum KeyUsage
          {
            DigitalSignature = 0,
            NonRepudiation,
            KeyEncipherment,
            DataEncipherment,
            KeyAgreement,
            KeyCertificateSign,
            CRLSign,
            EncipherOnly,
            DecipherOnly
          };

          X509Cert();
          X509Cert(X509* cert) throw(IOException);
          X509Cert(std::vector<unsigned char> bytes) throw(IOException);
          X509Cert(const X509Cert& copy) throw(IOException);
          ~X509Cert();
          X509Cert& operator=(const X509Cert& copy) throw(IOException);
          X509* getX509() const throw(IOException);
          static X509* copyX509(X509* cert) throw(IOException);
          static X509* loadX509(const std::string& path) throw(IOException);
          static STACK_OF(X509)* loadX509Stack(const std::string& path) throw(IOException);

          int getPaddingSize() const throw(IOException);
          std::vector<unsigned char> encodeDER() const throw(IOException);
          std::string getSerial() const throw(IOException);
          X509_NAME* getIssuerNameAsn1() const;
          std::string getIssuerName() const throw(IOException);
          std::vector<KeyUsage> getKeyUsage() const throw(IOException);
          std::vector<std::string> getCertificatePolicies() const throw(IOException);
          std::string getSubject() const throw(IOException);
          EVP_PKEY* getPublicKey() const throw(IOException);
          std::vector<unsigned char> getRsaModulus() const throw(IOException);
          std::vector<unsigned char> getRsaExponent() const throw(IOException);

          bool isValid() const throw(IOException);

          int verify() const throw(IOException);

          int compareIssuerToString(const std::string &name) const throw(IOException);

      private:
          std::string toString(X509_NAME *name) const throw(IOException);

          X509* cert;
    };
}

#endif // !defined(__X509CERT_H_INCLUDED__)
