#if !defined(__SIGNER_H_INCLUDED__)
#define __SIGNER_H_INCLUDED__

#include "../../Exception.h"
#include "../../SignatureAttributes.h"

#include <openssl/x509.h>

namespace digidoc
{
    /**
     * Signer interface. Provides methods for signing documents. Digidoc LIB implements EstID signer class
     * that allows signing with EstId chip card. Other implementation may provide signing implementation
     * with other public-key cryptography systems.
     */
    class EXP_DIGIDOC Signer
    {

      public:
          struct EXP_DIGIDOC Digest
          {
              /** Digest type (e.g NID_sha1), available values in openssl/obj_mac.h */
              int type;
              const unsigned char* digest;
              unsigned int length;
          };

          struct EXP_DIGIDOC Signature
          {
              unsigned char* signature;
              unsigned int length;
          };

          virtual int type() const;
          virtual X509* getCert() const throw(SignException) = 0;
          virtual void sign(const Digest& digest, Signature& signature) throw(SignException) = 0;
          void setSignatureProductionPlace(const SignatureProductionPlace& signatureProductionPlace);
          SignatureProductionPlace getSignatureProductionPlace() const;
          void setSignerRole(const SignerRole& signerRole);
          SignerRole getSignerRole() const;

      protected:
          virtual ~Signer() {}

          SignatureProductionPlace signatureProductionPlace;
          SignerRole signerRole;

    };
}

#endif // !defined(__SIGNER_H_INCLUDED__)
